#---balloonhelp
package provide FR_balloonHelp 0.2


namespace eval ::FR_balloonHelp \
{
 namespace export Activate Deactivate AddHelp
}


proc ::FR_balloonHelp::Activate {} \
{
 variable VAR; variable WIDGET_LIST
 
 if {![winfo exists .balloonhelp]} \
 {
   toplevel .balloonhelp -background black -borderwidth 1 -relief flat
   label .balloonhelp.info -background white -wraplength 3i -justify left
   
   pack .balloonhelp.info -side left -fill x
   wm overrideredirect .balloonhelp 1
   wm withdraw .balloonhelp
 } 

 # we collect all widgets with a help
 set WidgetListNames [array names WIDGET_LIST]
 
 if {[llength $WidgetListNames] == 0} \
 {
   return
 }
 
 # if there not yet a binding - we make it, but only if window exists yet
 foreach Arg $WidgetListNames \
 {
   if {[winfo exists $Arg]} \
   {
     if {![info exists VAR($Arg)]} \
     {
       bind $Arg <Enter> {::FR_balloonHelp::Pending %W}
       bind $Arg <Leave> {::FR_balloonHelp::Cancel}
   
       set VAR($Arg) $WIDGET_LIST($Arg)
     }
   }
 }
 
 array unset WIDGET_LIST
}


#---we hide the balloonhelp with unsetting the binding
proc ::FR_balloonHelp::Deactivate {} \
{ 
 variable VAR
 
 set WidgetListNames [array names VAR]
 
 if {[llength $WidgetListNames] == 0} \
 {
   return
 }
 
 foreach Arg $WidgetListNames \
 {
   if {[winfo exists $Arg]} \
   {
     bind $Arg <Enter> ""
     bind $Arg <Leave> ""
   }
 }
}


#---the user frontend to add a balloon message
proc ::FR_balloonHelp::AddHelp {Widget Message} \
{
 variable WIDGET_LIST
 
 set WIDGET_LIST($Widget) $Message
}


#---this removes the balloonhelp window
proc ::FR_balloonHelp::Cancel {} \
{
 variable PENDING_ID
 
 if { [info exists PENDING_ID] } \
 {
   after cancel $PENDING_ID
   unset PENDING_ID
 }
 
 wm withdraw .balloonhelp
}


#---if the mouse over a widget we will show the balloonhelp in 1.5 seconds
proc ::FR_balloonHelp::Pending {Widget} \
{
 variable PENDING_ID
 
 ::FR_balloonHelp::Cancel
 set PENDING_ID [after 1500 [list ::FR_balloonHelp::Show $Widget]] 
}


#---we bring the balloonhelp on the screen
proc ::FR_balloonHelp::Show {Widget} \
{
 variable PENDING_ID
 variable VAR
 
 .balloonhelp.info configure -text $VAR($Widget)
 
 set x [expr {[winfo rootx $Widget] + 10}]
 set y [expr {[winfo rooty $Widget] + [winfo height $Widget]}]
 
 update
 
 set BalloonHelpWidth [expr {$x + [winfo width .balloonhelp.info]}]
 
 if { $BalloonHelpWidth > [winfo screenwidth $Widget] } \
 {
   set x [expr {$x - [winfo width .balloonhelp.info]}]
 }
 
 wm geometry .balloonhelp +$x+$y
 wm deiconify .balloonhelp
 raise .balloonhelp
  
 # it could be that during executing ::FR_balloonHelp::Show
 # ::FR_balloonHelp::Cancel unset VAR(pending), so we ask for that
 if { [info exists PENDING_ID] } \
 {
   unset PENDING_ID
 }
}
