#---the usage of FR_comboBox:
#
# ::FR_comboBox::cb pathName ?options?
#
# Options:
#
# in the case of creation:
# -variable {the textvariable of the entry-widget}
# -command {a procedure which is called when a item from the listbox is selected}
#
# in the case of creation and/or filling:
# -listbox_content {the content of your popup listbox}
# -default_value {the value which is showed in the entry-box without a choice in the popup listbox}
# -entry_width {if you wish a fixed width of the entry-box in count of letters or set it to "auto";
#               in the "auto" case the width of the the entry-box is depending on the content of
#               the listbox (limited to 50 characters)}
#
# in the case configuration: 
# -conf_widget {entry | button | listbox}
# -conf_option {the same options as for a normal tk-widget specified with "-conf_widget"}
#
# to change the status:
# -state {normal | disable}
#
# to read out the content of the entry widget:
# -read


package provide FR_comboBox 0.2

namespace eval ::FR_comboBox \
{
  namespace export cb
}


proc ::FR_comboBox::cb {args} \
{ 
 variable VAR 
 
 # we read the pathname
 set Path ""
 set Path [lindex $args 0]

 # we check the correctness of the pathname
 if {$Path eq ""} \
 {
   puts "You must hand over a valid pathname, not an empty string"
   exit 1
 }
  
 set Path [string trimleft $Path .]
 

  
 # some presettings
 if {[winfo exists .$Path.cb_entry]} \
 {
   set VAR($Path,EntryWidth) [.$Path.cb_entry cget -width]
   # if the combobox already exists we set EntryExists to 1
   set VAR($Path,EntryExists) 1
 } else \
 {
   set VAR($Path,EntryWidth) 2
   set VAR($Path,EntryExists) 0
   
   # some default values
   set VAR($Path,ListBoxStatus) 0
   set VAR($Path,Parent) "."
   set VAR($Path,ReadCmd) 0
   
   foreach Arg [list ListBoxContent ListBoxDefault Variable Command Configure ConfigureOptions State] \
   {
     set VAR($Path,$Arg) ""
   }
 }
   
 # we check the args 
 for {set i 1} {$i < [llength $args]} {incr i 2} \
 {
   set Arg [lindex $args $i]

   switch -glob -- $Arg \
   { -entry_width     {set VAR($Path,EntryWidth) [lindex $args [expr {$i+1}]]} \
     -listbox_content {set VAR($Path,ListBoxContent) [lindex $args [expr {$i+1}]]} \
     -default_value   {set VAR($Path,ListBoxDefault) [lindex $args [expr {$i+1}]]} \
     -variable        {set VAR($Path,Variable) [lindex $args [expr {$i+1}]]} \
     -command         {set VAR($Path,Command) [lindex $args [expr {$i+1}]]} \
     -parent          {set VAR($Path,Parent) [lindex $args [expr {$i+1}]]} \
     -conf_widget     {set VAR($Path,Configure) [lindex $args [expr {$i+1}]]} \
     -conf_option     {set VAR($Path,ConfigureOptions) [lindex $args [expr {$i+1}]]} \
     -state           {set VAR($Path,State) [lindex $args [expr {$i+1}]]} \
     -read            {set VAR($Path,ReadCmd) 1} \
     --               {break} \
     default          {puts "Unknown option: $Arg"; exit 1} \
   }
 }


 # if the combox not yet exists -> we create one
 if {$VAR($Path,EntryExists) == 0} \
 {
  ::FR_comboBox::Create $Path
 }    

 # start the filling ("-listbox_content") of the combobox
 if {$VAR($Path,ListBoxContent) ne ""} \
 {
   ::FR_comboBox::FillListBox $Path
   
   set VAR($Path,ListBoxContent) "" 
 }

 # start the configuration ("-conf_widget" and "-conf_option") of the combobox 
 if {($VAR($Path,Configure) ne "") && ($VAR($Path,ConfigureOptions) ne "")} \
 {
   ::FR_comboBox::ConfigureComboBox $Path
   
   set VAR($Path,Configure) ""
 }
 
 # start the "-state" option as a subversion of configuration
 if {$VAR($Path,State) ne ""} \
 {
   # first we configure the button
   set VAR($Path,Configure) "button"
   set VAR($Path,ConfigureOptions) "-state $VAR($Path,State)"
  
   ::FR_comboBox::ConfigureComboBox $Path
  
   #...and now the entry widget
   set VAR($Path,Configure) "entry"
  
   if {$VAR($Path,State) eq "normal"} \
   {
     set VAR($Path,ConfigureOptions) "-disabledforeground black"
   } \
   else \
   {
     set VAR($Path,ConfigureOptions) "-disabledforeground grey"
   }
  
   ::FR_comboBox::ConfigureComboBox $Path
   
   set VAR($Path,State) ""
   set VAR($Path,Configure) ""
   set VAR($Path,ConfigureOptions) ""
 }
  
 # start the "-read" option
 if {$VAR($Path,ReadCmd)} \
 {
   set VAR($Path,ReadCmd) 0
   
   return [::FR_comboBox::Read $Path]
 }
 
 # if we have created a combobox in this run, we return the path
 if {$VAR($Path,EntryExists) == 0} \
 {
   return $Path
 } 
}


#---put a content in the entry widget
proc ::FR_comboBox::InsertEntry {Path Value} \
{
 .$Path.cb_entry configure -state normal
 .$Path.cb_entry delete 0 end
 .$Path.cb_entry insert end $Value
 .$Path.cb_entry configure -state disable
}


#---configure a combobox
proc ::FR_comboBox::ConfigureComboBox {Path} \
{
 variable VAR
   
 switch -glob -- $VAR($Path,Configure) \
 {
   "entry"   {set ConfigPath $Path.cb_entry} \
   "button"  {set ConfigPath $Path.cb_button} \
   "listbox" {set ConfigPath $Path.lb.cb_listbox} \
   default   {puts "Wrong widget chosen: $VAR($Path,Configure). Please use \"entry\", \"button\" or \"listbox\""; exit 1} \
 }
    
 if {[winfo exists .$ConfigPath]} \
 {  
   set ReturnValue [catch {eval .$ConfigPath configure $VAR($Path,ConfigureOptions)} CatchOutput]
   
   if {$ReturnValue != 0} {puts "Can't make this configuration. The reason is: $CatchOutput"}
 } \
 else \
 {
   puts "The $VAR($Path,Configure)-widget: \".$ConfigPath\" doesn't exists."
 }
}


#---read a combobox
proc ::FR_comboBox::Read {Path} \
{
 return [.${Path}.cb_entry get]
}


#---remove/add the x or y scrollbars
proc ::FR_comboBox::ScrollBarManager {ListBoxPath Orientation Action} \
{
 if {$Orientation eq "y"} \
 {
   set ScrollBarPath $ListBoxPath.cb_yscrollbar
   set GridCmd "grid .$ListBoxPath.cb_yscrollbar -row 0 -column 1 -sticky ns"
 } \
 elseif {$Orientation eq "x"} \
 {
   set ScrollBarPath $ListBoxPath.cb_xscrollbar
   set GridCmd "grid .$ListBoxPath.cb_xscrollbar -row 1 -column 0 -sticky ew -columnspan 2"
 }

 if { $Action eq "remove" } \
 {
   if {[winfo ismapped .$ListBoxPath]} {grid remove .$ScrollBarPath}
 } \
 elseif { $Action eq "view" } \
 {
   if {![winfo ismapped .$ListBoxPath]} {eval $GridCmd}
 }
}


#---we fill the listbox
proc ::FR_comboBox::FillListBox {Path} \
{
 variable VAR
    
 if {![winfo exists .$VAR($Path,ListBoxPath)]} \
 {
   puts "The \"$VAR($Path,ListBoxPath)\"-widget doesn't exists."
   exit 1  
 }
  
 # first we delete the old content
 .$VAR($Path,ListBoxPath).cb_listbox delete 0 end
  
 # now we fill the listbox with new content
 set MaxWidth 2; # we need MaxWidth for the width of the listbox
 set ListLenght [llength $VAR($Path,ListBoxContent)]
  
 if { $ListLenght >= 1 } \
 {
   foreach Arg $VAR($Path,ListBoxContent) \
   {
     .$VAR($Path,ListBoxPath).cb_listbox insert end $Arg
     set Count [string length $Arg]
     if {$Count > $MaxWidth} {set MaxWidth $Count}
   }
   
   # for count of list elements < 8 we adapt the height of the listbox, in the other case we set the height to 8
   if { $ListLenght <= 8} \
   {
     .$VAR($Path,ListBoxPath).cb_listbox configure -height $ListLenght
       
     ::FR_comboBox::ScrollBarManager $VAR($Path,ListBoxPath) y remove
   } \
   else \
   {
     .$VAR($Path,ListBoxPath).cb_listbox configure -height 8
     ::FR_comboBox::ScrollBarManager $VAR($Path,ListBoxPath) y view
   }
 }
   
 # the auto-entry-width function, EntryWidthAuto comes from the creation of the combobox
 if {($VAR($Path,EntryWidth) eq "auto") || $VAR($Path,EntryWidthAuto)} \
 {
   # we set the entry width to the maximum count of letters and/or limit it to 50 letters
   if {$MaxWidth < 50} \
   {
     set VAR($Path,EntryWidth) $MaxWidth
   } \
   else \
   {
     set VAR($Path,EntryWidth) 50
   }
 }
   
 # we view or remove the x-axis scrollbar
 if { $MaxWidth <= $VAR($Path,EntryWidth)} \
 {
   ::FR_comboBox::ScrollBarManager $VAR($Path,ListBoxPath) x remove
 } \
 else \
 {
   ::FR_comboBox::ScrollBarManager $VAR($Path,ListBoxPath) x view
 }
  
 # we set the marked element in the listbox to the LB_DEFAULT or to the first element in the listbox-content
 set SearchResult "-1"
   
 if {[llength $VAR($Path,ListBoxDefault)] > 0} \
 {
   set SearchResult [lsearch -exact $VAR($Path,ListBoxContent) $VAR($Path,ListBoxDefault)]
 }
   
 if { $SearchResult == -1} \
 {
   set SearchResult 0
 }
  
 .$VAR($Path,ListBoxPath).cb_listbox selection set $SearchResult
 .$VAR($Path,ListBoxPath).cb_listbox see $SearchResult
  
 ::FR_comboBox::InsertEntry $Path [lindex $VAR($Path,ListBoxContent) $SearchResult]
    
    
 .$Path.cb_entry configure -width $VAR($Path,EntryWidth)
}


#---create a combobox
proc ::FR_comboBox::Create {Path} \
{
 variable VAR
 variable ButtonImage
  
 # set the default for the -variable option
 if {$VAR($Path,Variable) eq ""} {set VAR($Path,Variable) "noname"}
  
 # we check if the "-variable" parameter a array and make the arrayname to a global variable
 if {[regexp {([^(]*)([(]+)([^)]*)([)]+)} $VAR($Path,Variable) All Arrayname All Tail All]} \
 {
   global [set Arrayname]
   
   unset All Tail
 } \
 else \
 {
   # if the "-variable" parameter a normal variable we make it to a global variable
   global [set VAR($Path,Variable)]
 }
   
 if {[winfo exists .$Path]} \
 {
   puts "The widget with the path \".$Path\" already exits. Please choice an other name."
 }
    
 frame .$Path -relief flat
 pack .$Path
    
 set VAR($Path,ListBoxPath) "$Path.lb"
 set VAR($Path,ListBoxStatus) 0
   
 if {[winfo exists .$VAR($Path,ListBoxPath)]} \
 {
   puts "The widget with the path \".$VAR($Path,ListBoxPath)\" already exits. Please choice an other name."
 }
  
 # safe the value "auto" if set with -entry-width option
 if {$VAR($Path,EntryWidth) eq "auto"} \
 {
   set VAR($Path,EntryWidthAuto) 1
   # we set EntryWidth temporaly to 2 to have an integer variable and not string "auto"
   set VAR($Path,EntryWidth) 2
 } \
 else \
 {
   set VAR($Path,EntryWidthAuto) 0
 }
  
 # the image for the button
 if {![info exists ButtonImage]} \
 {
   set ButtonImage [image create bitmap -data {#define combobox_width 16 #define combobox_height 16
   static unsigned char combobox_bits[] = {
   0x00, 0x00, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01,
   0x80, 0x01, 0x90, 0x09, 0xb0, 0x0d, 0xe0, 0x07, 0xc0, 0x03, 0x80, 0x01,
   0x00, 0x00, 0xf0, 0x0f, 0xf0, 0x0f, 0x00, 0x00};}]
 }
  
 # we create a new topwindow for the listbox
 toplevel .$VAR($Path,ListBoxPath) -borderwidth 2 -relief groove
   
 listbox .$VAR($Path,ListBoxPath).cb_listbox -activestyle none -yscroll ".$VAR($Path,ListBoxPath).cb_yscrollbar set" -xscroll ".$VAR($Path,ListBoxPath).cb_xscrollbar set" -background white
  
 scrollbar .$VAR($Path,ListBoxPath).cb_yscrollbar -orient vertical -command ".$VAR($Path,ListBoxPath).cb_listbox yview"
 scrollbar .$VAR($Path,ListBoxPath).cb_xscrollbar -orient horizontal -command ".$VAR($Path,ListBoxPath).cb_listbox xview"
   
 grid .$VAR($Path,ListBoxPath).cb_listbox -row 0 -column 0 -sticky nsew
 grid columnconfigure .$VAR($Path,ListBoxPath) 0 -weight 1
  
 wm overrideredirect .$VAR($Path,ListBoxPath) 1
 wm withdraw .$VAR($Path,ListBoxPath)
  

 eval entry .$Path.cb_entry -disabledforeground black -disabledbackground white -width $VAR($Path,EntryWidth) -state disable -textvariable [set VAR($Path,Variable)]
 button .$Path.cb_button -image $ButtonImage
  
 pack .$Path.cb_entry -side left -fill x -expand yes 
 pack .$Path.cb_button -side right
  
 bind .$Path.cb_button <ButtonPress-1> {::FR_comboBox::ButtonAction %W}  
 bind .$VAR($Path,ListBoxPath).cb_listbox <<ListboxSelect>> {::FR_comboBox::ListBoxSelection %W}
  
 if {[winfo exists $VAR($Path,Parent)]} \
 {
   bindtags $VAR($Path,Parent) [linsert [bindtags $VAR($Path,Parent)] 0 ConfigureRefresh]
 } \
 else \
 {
   puts "I didn't find the \"-parent\" window: $VAR($Path,Parent)"
 }
 
 bind ConfigureRefresh <Configure> {::FR_comboBox::RemoveAllListBox}

}

 
#---a subproc which execute the -command parameter after a selection in the listbox
proc ::FR_comboBox::ExecuteCmd {Path} \
{
 variable VAR
 #puts $VAR($Path,Command) 
 if {$VAR($Path,Command) ne ""} \
 {
   eval $VAR($Path,Command)
   #catch [set VAR($Path,Command)]
 }
}
  
  
#---a subproc which calculate the size of the listbox and pop it up
proc ::FR_comboBox::ListBoxPopup {Path} \
{
 variable VAR
   
 set PosX [winfo rootx .$Path.cb_entry]
 set PosY [expr {[winfo rooty .$Path.cb_entry] + [winfo height .$Path.cb_button]}]
   
 set WidthX [expr {[winfo width .$Path.cb_entry] + [winfo width .$Path.cb_button] - [.$Path.cb_button cget -borderwidth]}]
 set HeightY [winfo reqheight .$VAR($Path,ListBoxPath)]
   
 wm geometry .$VAR($Path,ListBoxPath) ${WidthX}x${HeightY}+${PosX}+${PosY}
 wm deiconify .$VAR($Path,ListBoxPath)
 raise .$VAR($Path,ListBoxPath)
   
 set VAR($Path,ListBoxStatus) 1
}
  
  
#---a subproc which close the listbox
proc ::FR_comboBox::ListBoxRemove {Path} \
{
 variable VAR
   
 wm withdraw .$VAR($Path,ListBoxPath)
   
 set VAR($Path,ListBoxStatus) 0
}
   
  
#---the action after pressing the button 
proc ::FR_comboBox::ButtonAction {Path} \
{
 variable VAR
   
 # we do it only if button is in the normal state
 if {[$Path cget -state] ne "active"} {return}
   
 set Path [winfo parent $Path]
 set Path [string trimleft $Path .]
   
 if {$VAR($Path,ListBoxStatus)} \
 {
   ::FR_comboBox::ListBoxRemove $Path
 } \
 else \
 {
   ::FR_comboBox::ListBoxPopup $Path
 }
}
   
  
#---the action after selection a item in the listbox
proc ::FR_comboBox::ListBoxSelection {Path} \
{
 set Index [$Path curselection] 
 set Selected [$Path get $Index]
   
 set Path [winfo parent [winfo parent $Path]]
 set Path [string trimleft $Path .]
   
 # we remove the listbox, put the selection in the entry widget and execute the -command argument
 ::FR_comboBox::ListBoxRemove $Path
   
 ::FR_comboBox::InsertEntry $Path $Selected
   
 ::FR_comboBox::ExecuteCmd $Path
}


#---if . configures we remove all open listboxes
proc ::FR_comboBox::RemoveAllListBox {} \
{
 variable VAR
 
 set AllComboBoxes ""
 set Tmp ""
 
 # first we search all comboboxes
 if {[array exists VAR]} \
 {
   set AllComboBoxes [array names VAR "*,ListBoxStatus"]
   
   foreach Arg $AllComboBoxes \
   {
     set Index [expr {[string first "," $Arg] - 1}]
     lappend Tmp [string range $Arg 0 $Index]
   }
  
   set AllComboBoxes $Tmp
 }
 
 # now we close all open comboboxes
 if {[llength $AllComboBoxes] > 0} \
 {
   foreach Arg $AllComboBoxes \
   {
     if {$VAR($Arg,ListBoxStatus)} \
     {
       wm withdraw .$VAR($Arg,ListBox)
       set VAR($Arg,ListBoxStatus) 0
     }
   }
 } \
 else \
 {
   puts "No comboboxes found"
 }
}
